@@ -4,14 +4,14 @@
  */
 
 #include "libbb.h"
-#include "archive.h"
+#include "bb_archive.h"
 
 char FAST_FUNC get_header_tar_bz2(archive_handle_t *archive_handle)
 {
 	/* Can't lseek over pipes */
 	archive_handle->seek = seek_by_read;
 
-	open_transformer(archive_handle->src_fd, unpack_bz2_stream_prime, "bunzip2");
+	open_transformer_with_sig(archive_handle->src_fd, unpack_bz2_stream, "bunzip2");
 	archive_handle->offset = 0;
 	while (get_header_tar(archive_handle) == EXIT_SUCCESS)
 		continue;
