@@ -246,10 +246,21 @@ enum speed_setting {
 
 /* Which member(s) of 'struct termios' a mode uses */
 enum {
-	/* Do NOT change the order or values, as mode_type_flag()
-	 * depends on them */
 	control, input, output, local, combination
 };
+static tcflag_t *get_ptr_to_tcflag(unsigned type, const struct termios *mode)
+{
+	static const uint8_t tcflag_offsets[] ALIGN1 = {
+		offsetof(struct termios, c_cflag), /* control */
+		offsetof(struct termios, c_iflag), /* input */
+		offsetof(struct termios, c_oflag), /* output */
+		offsetof(struct termios, c_lflag)  /* local */
+	};
+	if (type <= local) {
+		return (tcflag_t*) (((char*)mode) + tcflag_offsets[type]);
+	}
+	return NULL;
+}
 
 /* Flags for 'struct mode_info' */
 #define SANE_SET 1              /* Set in 'sane' mode                  */
@@ -770,51 +781,6 @@ struct globals {
 	G.max_col = 80; \
 } while (0)
 
-
-/* Return a string that is the printable representation of character CH */
-/* Adapted from 'cat' by Torbjorn Granlund */
-static const char *visible(unsigned ch)
-{
-	char *bpout = G.buf;
-
-	if (ch == _POSIX_VDISABLE)
-		return "<undef>";
-
-	if (ch >= 128) {
-		ch -= 128;
-		*bpout++ = 'M';
-		*bpout++ = '-';
-	}
-
-	if (ch < 32) {
-		*bpout++ = '^';
-		*bpout++ = ch + 64;
-	} else if (ch < 127) {
-		*bpout++ = ch;
-	} else {
-		*bpout++ = '^';
-		*bpout++ = '?';
-	}
-
-	*bpout = '\0';
-	return G.buf;
-}
-
-static tcflag_t *mode_type_flag(unsigned type, const struct termios *mode)
-{
-	static const uint8_t tcflag_offsets[] ALIGN1 = {
-		offsetof(struct termios, c_cflag), /* control */
-		offsetof(struct termios, c_iflag), /* input */
-		offsetof(struct termios, c_oflag), /* output */
-		offsetof(struct termios, c_lflag)  /* local */
-	};
-
-	if (type <= local) {
-		return (tcflag_t*) (((char*)mode) + tcflag_offsets[type]);
-	}
-	return NULL;
-}
-
 static void set_speed_or_die(enum speed_setting type, const char *arg,
 					struct termios *mode)
 {
@@ -1042,6 +1008,7 @@ static void do_display(const struct termios *mode, int all)
 #endif
 
 	for (i = 0; i != CIDX_min; ++i) {
+		char ch;
 		/* If swtch is the same as susp, don't print both */
 #if VSWTCH == VSUSP
 		if (i == CIDX_swtch)
@@ -1055,8 +1022,12 @@ static void do_display(const struct termios *mode, int all)
 			continue;
 		}
 #endif
-		wrapf("%s = %s;", nth_string(control_name, i),
-				visible(mode->c_cc[control_info[i].offset]));
+		ch = mode->c_cc[control_info[i].offset];
+		if (ch == _POSIX_VDISABLE)
+			strcpy(G.buf, "<undef>");
+		else
+			visible(ch, G.buf, 0);
+		wrapf("%s = %s;", nth_string(control_name, i), G.buf);
 	}
 #if VEOF == VMIN
 	if ((mode->c_lflag & ICANON) == 0)
@@ -1072,7 +1043,7 @@ static void do_display(const struct termios *mode, int all)
 			prev_type = mode_info[i].type;
 		}
 
-		bitsp = mode_type_flag(mode_info[i].type, mode);
+		bitsp = get_ptr_to_tcflag(mode_info[i].type, mode);
 		mask = mode_info[i].mask ? mode_info[i].mask : mode_info[i].bits;
 		if ((*bitsp & mask) == mode_info[i].bits) {
 			if (all || (mode_info[i].flags & SANE_UNSET))
@@ -1091,7 +1062,6 @@ static void do_display(const struct termios *mode, int all)
 static void sane_mode(struct termios *mode)
 {
 	int i;
-	tcflag_t *bitsp;
 
 	for (i = 0; i < NUM_control_info; ++i) {
 #if VMIN == VEOF
@@ -1102,14 +1072,17 @@ static void sane_mode(struct termios *mode)
 	}
 
 	for (i = 0; i < NUM_mode_info; ++i) {
+		tcflag_t val;
+		tcflag_t *bitsp = get_ptr_to_tcflag(mode_info[i].type, mode);
+
+		if (!bitsp)
+			continue;
+		val = *bitsp & ~((unsigned long)mode_info[i].mask);
 		if (mode_info[i].flags & SANE_SET) {
-			bitsp = mode_type_flag(mode_info[i].type, mode);
-			*bitsp = (*bitsp & ~((unsigned long)mode_info[i].mask))
-				| mode_info[i].bits;
-		} else if (mode_info[i].flags & SANE_UNSET) {
-			bitsp = mode_type_flag(mode_info[i].type, mode);
-			*bitsp = *bitsp & ~((unsigned long)mode_info[i].mask)
-				& ~mode_info[i].bits;
+			*bitsp = val | mode_info[i].bits;
+		} else
+		if (mode_info[i].flags & SANE_UNSET) {
+			*bitsp = val & ~mode_info[i].bits;
 		}
 	}
 }
@@ -1119,17 +1092,18 @@ static void set_mode(const struct mode_info *info, int reversed,
 {
 	tcflag_t *bitsp;
 
-	bitsp = mode_type_flag(info->type, mode);
+	bitsp = get_ptr_to_tcflag(info->type, mode);
 
 	if (bitsp) {
+		tcflag_t val = *bitsp & ~info->mask;
 		if (reversed)
-			*bitsp = *bitsp & ~info->mask & ~info->bits;
+			*bitsp = val & ~info->bits;
 		else
-			*bitsp = (*bitsp & ~info->mask) | info->bits;
+			*bitsp = val | info->bits;
 		return;
 	}
 
-	/* Combination mode */
+	/* !bitsp - it's a "combination" mode */
 	if (info == &mode_info[IDX_evenp] || info == &mode_info[IDX_parity]) {
 		if (reversed)
 			mode->c_cflag = (mode->c_cflag & ~PARENB & ~CSIZE) | CS8;
@@ -1534,7 +1508,12 @@ int stty_main(int argc UNUSED_PARAM, char **argv)
 			perror_on_device_and_die("%s");
 
 		if (memcmp(&mode, &new_mode, sizeof(mode)) != 0) {
-#if CIBAUD
+/*
+ * I think the below chunk is not necessary on Linux.
+ * If you are deleting it, also delete STTY_speed_was_set bit -
+ * it is only ever checked here.
+ */
+#if 0 /* was "if CIBAUD" */
 			/* SunOS 4.1.3 (at least) has the problem that after this sequence,
 			   tcgetattr (&m1); tcsetattr (&m1); tcgetattr (&m2);
 			   sometimes (m1 != m2).  The only difference is in the four bits
