@@ -34,6 +34,19 @@
  * It doesn't guess filesystem types from on-disk format.
  */
 
+//usage:#define fsck_trivial_usage
+//usage:       "[-ANPRTV] [-C FD] [-t FSTYPE] [FS_OPTS] [BLOCKDEV]..."
+//usage:#define fsck_full_usage "\n\n"
+//usage:       "Check and repair filesystems\n"
+//usage:     "\n	-A	Walk /etc/fstab and check all filesystems"
+//usage:     "\n	-N	Don't execute, just show what would be done"
+//usage:     "\n	-P	With -A, check filesystems in parallel"
+//usage:     "\n	-R	With -A, skip the root filesystem"
+//usage:     "\n	-T	Don't show title on startup"
+//usage:     "\n	-V	Verbose"
+//usage:     "\n	-C n	Write status information to specified filedescriptor"
+//usage:     "\n	-t TYPE	List of filesystem types to check"
+
 #include "libbb.h"
 
 /* "progress indicator" code is somewhat buggy and ext[23] specific.
@@ -303,7 +316,6 @@ static void load_fs_info(const char *filename)
 {
 	FILE *fstab;
 	struct mntent mte;
-	struct fs_info *fs;
 
 	fstab = setmntent(filename, "r");
 	if (!fstab) {
@@ -316,7 +328,7 @@ static void load_fs_info(const char *filename)
 		//bb_info_msg("CREATE[%s][%s][%s][%s][%d]", mte.mnt_fsname, mte.mnt_dir,
 		//	mte.mnt_type, mte.mnt_opts,
 		//	mte.mnt_passno);
-		fs = create_fs_device(mte.mnt_fsname, mte.mnt_dir,
+		create_fs_device(mte.mnt_fsname, mte.mnt_dir,
 			mte.mnt_type, mte.mnt_opts,
 			mte.mnt_passno);
 	}
@@ -466,7 +478,7 @@ static int wait_one(int flags)
 		instance_list = inst->next;
 	if (verbose > 1)
 		printf("Finished with %s (exit status %d)\n",
-		       inst->device, status);
+			inst->device, status);
 	num_running--;
 	free_instance(inst);
 
@@ -832,7 +844,7 @@ static int check_all(void)
 		if (verbose > 1)
 			printf("--waiting-- (pass %d)\n", passno);
 		status |= wait_many(pass_done ? FLAG_WAIT_ALL :
-				    FLAG_WAIT_ATLEAST_ONE);
+				FLAG_WAIT_ATLEAST_ONE);
 		if (pass_done) {
 			if (verbose > 1)
 				puts("----------------------------------");
