@@ -1,21 +1,26 @@
 /* vi: set sw=4 ts=4: */
 /*
- * Licensed under the GPL v2 or later, see the file LICENSE in this tarball.
+ * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
 
+//usage:#define dumpleases_trivial_usage
+//usage:       "[-r|-a] [-f LEASEFILE]"
+//usage:#define dumpleases_full_usage "\n\n"
+//usage:       "Display DHCP leases granted by udhcpd\n"
+//usage:	IF_LONG_OPTS(
+//usage:     "\n	-f,--file=FILE	Lease file"
+//usage:     "\n	-r,--remaining	Show remaining time"
+//usage:     "\n	-a,--absolute	Show expiration time"
+//usage:	)
+//usage:	IF_NOT_LONG_OPTS(
+//usage:     "\n	-f FILE	Lease file"
+//usage:     "\n	-r	Show remaining time"
+//usage:     "\n	-a	Show expiration time"
+//usage:	)
+
 #include "common.h"
 #include "dhcpd.h"
-
-#if BB_LITTLE_ENDIAN
-static inline uint64_t hton64(uint64_t v)
-{
-        return (((uint64_t)htonl(v)) << 32) | htonl(v >> 32);
-}
-#else
-#define hton64(v) (v)
-#endif
-#define ntoh64(v) hton64(v)
-
+#include "unicode.h"
 
 int dumpleases_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int dumpleases_main(int argc UNUSED_PARAM, char **argv)
@@ -29,9 +34,9 @@ int dumpleases_main(int argc UNUSED_PARAM, char **argv)
 	struct in_addr addr;
 
 	enum {
-		OPT_a	= 0x1,	// -a
-		OPT_r	= 0x2,	// -r
-		OPT_f	= 0x4,	// -f
+		OPT_a = 0x1, // -a
+		OPT_r = 0x2, // -r
+		OPT_f = 0x4, // -f
 	};
 #if ENABLE_LONG_OPTS
 	static const char dumpleases_longopts[] ALIGN1 =
@@ -42,6 +47,8 @@ int dumpleases_main(int argc UNUSED_PARAM, char **argv)
 
 	applet_long_options = dumpleases_longopts;
 #endif
+	init_unicode();
+
 	opt_complementary = "=0:a--r:r--a";
 	opt = getopt32(argv, "arf:", &file);
 
@@ -51,9 +58,8 @@ int dumpleases_main(int argc UNUSED_PARAM, char **argv)
 	/*     "00:00:00:00:00:00 255.255.255.255 ABCDEFGHIJKLMNOPQRS Wed Jun 30 21:49:08 1993" */
 	/*     "123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 */
 
-	if (full_read(fd, &written_at, sizeof(written_at)) != sizeof(written_at))
-		return 0;
-	written_at = ntoh64(written_at);
+	xread(fd, &written_at, sizeof(written_at));
+	written_at = SWAP_BE64(written_at);
 	curr = time(NULL);
 	if (curr < written_at)
 		written_at = curr; /* lease file from future! :) */
@@ -65,9 +71,17 @@ int dumpleases_main(int argc UNUSED_PARAM, char **argv)
 			fmt = ":%02x";
 		}
 		addr.s_addr = lease.lease_nip;
+#if ENABLE_UNICODE_SUPPORT
+		{
+			char *uni_name = unicode_conv_to_printable_fixedwidth(/*NULL,*/ lease.hostname, 19);
+			printf(" %-16s%s ", inet_ntoa(addr), uni_name);
+			free(uni_name);
+		}
+#else
 		/* actually, 15+1 and 19+1, +1 is a space between columns */
 		/* lease.hostname is char[20] and is always NUL terminated */
 		printf(" %-16s%-20s", inet_ntoa(addr), lease.hostname);
+#endif
 		expires_abs = ntohl(lease.expires) + written_at;
 		if (expires_abs <= curr) {
 			puts("expired");
